//#include "add.h"
#include "Bitmap.h"
#include "Font.h"

#include <iostream>
#include <internal.hpp>

bool isQuit(std::string command);

/**
 * @author: Julian Hinxlage
 * @since: v0.0.0
 * @brief: An image is loaded and used as a font.
 * A string is converted and displayed in the console.
 * Currently only to test.
 */

#include "TextRenderer.h"
void outputWindow(int windowHeight, int windowWidth, vkvm::Color fontColor);
TextRenderer generateTextRender();
void renderText(TextRenderer textRenderer);
void test();

void test(TextRenderer renderer);

void redraw(TextRenderer renderer);

void test1();

int main(int argc, char *argv[]) {
    vkvm::initialize(0);

    TextRenderer textRenderer = generateTextRender();

    textRenderer.clear();

//    test(textRenderer);

    vkvm::registerEvent(vkvm::EventType::RenderText, [&textRenderer]() {
        renderText(textRenderer);
    });

//    test1();

    return 0;
}

void redraw(TextRenderer textRenderer) {
    textRenderer.clear();
    textRenderer.update(vkvm::getText());
}

void renderText(TextRenderer textRenderer) {
    textRenderer.update(vkvm::getText());
}

TextRenderer generateTextRender() {
    std::string command;
    std::cout << "fontSize: ";
    std::getline(std::cin, command);

    int windowWidth = vkvm::getWidth();
    int windowHeight = vkvm::getHeight();
    vkvm::Color fontColor = vkvm::getForegroundColor();
    vkvm::Color backgroundColor = vkvm::getBackgroundColor();

    vkvm::FontType fontType = vkvm::getFont();
    int fontSize = std::stoi(command);

    int font_id = fontType.getId();
    std::string fontResourcePath = "../res/font" + std::to_string(3)  + ".bmp";
    std::string fontConfigureFilePath = "../res/font" + std::to_string(3)  + ".toml";
    Font font = Font(fontResourcePath, fontConfigureFilePath);

    return TextRenderer(windowWidth, windowHeight, backgroundColor, fontColor, font, fontSize);
}

void test1() {
    std::string command;
    std::cout << "TextRender: ";
    std::getline(std::cin, command);
    while(command.compare("quit") != 0) {
        if(command.compare("RenderText") == 0) {
            vkvm::callEvent(vkvm::EventType::RenderText);
        }

        else if(command.compare("Redraw") == 0) {
            vkvm::callEvent(vkvm::EventType::Redraw);
        }

        else {
            vkvm::setText(command);
        }
    }

    std::cout << "TextRender finished." << std::endl;}

void test(TextRenderer renderer) {
    std::string currentText;

    std::string command;
    std::cout << "TextRender: ";
    std::getline(std::cin, command);
    while(!isQuit(command)) {
        if(command.compare("clear")) {
            vkvm::setText(command);
            currentText = vkvm::getText();
            renderer.update(currentText);
        } else {
            renderer.clear();
        }

        outputWindow(renderer.getWindowHeight(), renderer.getWindowWidth(), vkvm::getForegroundColor());
        std::cout << "TextRender: ";
        std::getline(std::cin, command);
    }

    std::cout << "TextRender finished." << std::endl;
}

void test() {
    std::string currentText;

    /*************************set back to shared memory(only for test)********************************************/
    int windowWidth = 40;
    int windowHeight = 15;
    vkvm::setWidth(windowWidth);
    vkvm::setHeight(windowHeight);

    vkvm::Color fontColor1(0, 0, 0);
    vkvm::Color backgroudColor1(255, 255, 255);
    Font font1("../res/font3.bmp", "../res/font3.toml");
    vkvm::setFont(vkvm::FontType(3, "font", font1.height(), font1.width()));
    vkvm::setForegroundColor(fontColor1);
    vkvm::setBackgroundColor(backgroudColor1);
    /**************************get text and font from shared memory*******************************************/
    std::string fontResourcePath = "../res/font" + std::to_string(vkvm::getFont().getId())  + ".bmp";
    std::string fontConfigureFilePath = "../res/font" + std::to_string(vkvm::getFont().getId())  + ".toml";
    Font font = Font(fontResourcePath, fontConfigureFilePath);
    vkvm::Color fontColor = vkvm::getForegroundColor();
    vkvm::Color backgroundColor = vkvm::getBackgroundColor();
    TextRenderer textRenderer(windowWidth, windowHeight, backgroundColor, fontColor, font);
    textRenderer.setLeftMargin(1);
    textRenderer.setBottomMargin(1);
    /*************************get Text and update back to shared meomory********************************************/

    std::string command;
    std::cout << "TextRender: ";
    std::getline(std::cin, command);
    while(!isQuit(command)) {
        if(command.compare("clear")) {
            vkvm::setText(command);
            currentText = vkvm::getText();
            textRenderer.update(currentText);
            outputWindow(windowHeight, windowWidth, fontColor);
            std::cout << "TextRender: ";
            std::getline(std::cin, command);
        } else {
            textRenderer.clear();
        }
    }

    std::cout << "TextRender finished." << std::endl;
}


/***************************read pixel in shared memory and test output in console******************************************/
void outputWindow(int windowHeight, int windowWidth, vkvm::Color fontColor) {
    for(int y = 0; y < windowHeight; y++) {
        for(int x = 0; x < windowWidth; x++) {
            if(vkvm::getPixel(x, y).getRed() == fontColor.getRed()) {
                std::cout << "*";
            } else {
                std::cout << " ";
            }
        }
        std::cout << "\n";
    }
}

bool isQuit(std::string command) {
    return command.compare("quit") == 0;
}
